//                        align2dummy.C
//   change all : , * to dummy in align statement
//  written by Zheng Gengbin

/*
	After align2dummy pass , we can assume :
	1. the align sources have only dummy , no : or * presents .
	2. replace the : of the align with subscript to the expression
	   of corresponding dummy . the formula is :
		( Dummy - LA )*ST+LT
	   among which , align-source array have bounds of (LA ,UA) ,
	   the corresponding subscript triple is LT:UT:ST .
	3. the * of align with subscript is left untouched .
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#include <strings.h>
#endif

//#include <sys/time.h>

#include "compilerApi.h"
#include "pass.h"


//
//generate a temporary variable by given type and in given statement block
//
SgSymbol *get_temp_variable(SgStatement *hedr_stmt,char *idstr)
{       
	char    str[60];
        SgSymbol  *sym;
        SgStatement     *last_decl=hedr_stmt->lastDeclaration();
//	struct timeval   time;
	int counter=0;
/*	   another strategy
	gettimeofday(&time,NULL);
        sprintf(str,"%s%d",idstr,time.tv_usec%10000);
*/
	while (1)
	{
        	sprintf(str,"%s%03d",idstr,counter);
		if ( !((SgProgHedrStmt *)hedr_stmt)->isSymbolNameInScope(str)) break;
		counter++;
	}
        //sym=new SgVariableSymb(str,*SgTypeInt(),*hedr_stmt);
	sym=hedr_stmt->GetSymbolInScope(hedr_stmt,str);
	//sym->setNextInScope(hedr_stmt);
        return(sym);
}

void trans_align_colon_to_dummy(SgStatement *s)
{
	SgSymbol     *dummy,*align_target,*alignee;
	SgExpression *align_op,*align_source,*align_with,*source,*with,*sele,*wele;
	SgExpression *expr;
	SgArrayType  *arraytype ;
	int dim1,dim2;
	SgStatement *hedr_stmt = s->getScopeForDeclare();

	align_op = s->expr(2)->lhs();
	align_source=source=align_op->lhs();
	align_with=with=align_op->rhs()->rhs()->lhs();

	align_target = align_op->rhs()->rhs()->symbol();
	alignee= s->expr(0)->lhs()->symbol();

	dim1=1; dim2=1;
	while (source || with )
	{
	  while (source && source->lhs()->variant() != COLON_OP )
	  {
		if (source->lhs()->variant() == STAR_OP)
		{
			dummy=get_temp_variable(hedr_stmt,"DM");
			expr=new SgVarRefExp(dummy);
			source->setLhs(expr);
		}
              	source = source->rhs(); dim1++;
       	  }
	  sele=source?source->lhs():(SgExpression *)NULL;

	  while (with && with->lhs()->variant() != DDOT )
	  {
		if (with->lhs()->variant() == STAR_OP)
		{
/*
			dummy=get_temp_variable(hedr_stmt,"DM");
			expr=new SgVarRefExp(dummy);
			with->setLhs(expr);
*/
		}
             	with = with->rhs(); dim2++;
       	  }
	  wele=with?with->lhs():(SgExpression *)NULL;

	  if (!source && !with ) break;

	  if (sele) 
	  {
	    if ( sele->variant() != COLON_OP)          // must be a dummy
	    {
 	    }
	    else            // COLON_OP
	    {
		  // declare a dummy and replace COLON_OP
		dummy=get_temp_variable(hedr_stmt,"DM");
		SgExpression *expr=new SgVarRefExp(dummy);
		source->setLhs(expr);
		  // figure out the expression of the align subscript and replace it.
		SgSubscriptExp *ddot=(SgSubscriptExp*)wele;
		SgExpression *J=new SgVarRefExp(dummy);
		SgExpression *ST=ddot->step();
		SgExpression *LT=ddot->lhs();
		if (LT) LT=LT->lhs();
		if (LT==NULL)
		{
			arraytype = (SgArrayType*)align_target->type();
			LT= ((SgSubscriptExp *)(arraytype->sizeInDim(dim2-1)))->lbound();
			if (LT==NULL)  LT=new SgValueExp(1); 
		}
		SgExpression *LA=get_lbound(alignee,dim1-1);

		  //  now change the rhs of align subscript
		with->setLhs((*J-*LA)*(*ST)+(*LT));
	    }
	  }  // end of if (sele)

	  if (source) { source=source->rhs(); dim1++; }
	  if (with)   { with=with->rhs(); dim2++; }
	}
}

